using System.Collections;
using System.Collections.Generic;
using UnityEngine;

// Matrix is used in DirectX and OpenGL, and also used in Unity shader code
public class VectorTest : MonoBehaviour
{
    public static float Theta(Vector3 v1,  Vector3 v2)
    {
	    return Mathf.Acos(Vector3.Dot(v1.normalized, v2.normalized));
    }

    public static Vector3 ComputeNormal(Vector3 a, Vector3 b, Vector3 c)
    {
	    a = Vector3.Normalize(a);
	    b = Vector3.Normalize(b);
	    c = Vector3.Normalize(c);
	    Vector3 normal = Vector3.Normalize(Vector3.Cross(c - a, b - a));
	    return normal;
    }

    private void Awake()
    {
        Debug.Log("VectorTest Awake");
        Vector3 a = new Vector3(0.0f, 0.0f, 0.0f);
        Vector3 b = new Vector3(1.0f, 2.0f, 3.0f);
        Vector3 c = b;
        Vector3 d = c;
        Debug.Log("a=" + a);
        Debug.Log("b=" + b);
        Debug.Log("c=" + c);
        Debug.Log("d=" + d);

        Debug.Log("VectorTest Unary Operation");
        a += b;
        Debug.Log("a+=b a=" + a);
        a *= 1.5f;
        Debug.Log("a*=1.5 a=" + a);
        a /= 1.5f;
        Debug.Log("a/=1.5 a=" + a);
        a -= b;
        Debug.Log("a-=b a=" + a);

        Debug.Log("VectorTest Binary Operation");
        c = a + b;
        Debug.Log("c=a+b c=" + c);
        c = a - b;
        Debug.Log("c=a-b c=" + c);

        Debug.Log("a=" + a);
        Debug.Log("b=" + b);
        if (a == b)
            Debug.Log("a == b");
        else
            Debug.Log("a != b");

        Debug.Log("b=" + b);
        Debug.Log("d=" + d);
        if (b == d)
            Debug.Log("b == d");
        else
            Debug.Log("b != d");

        // magnitude
        Debug.Log("a magnitude =" + a.magnitude);
        Debug.Log("b magnitude =" + b.magnitude);
	
        // normalize
        Debug.Log("c=" + c);
        Debug.Log("c normalize =" + c.normalized);
        c = Vector3.Normalize(c);
        Debug.Log("c normalize =" + c);
        Debug.Log("d=" + d);
        Debug.Log("d normalize =" + d.normalized);
        d = Vector3.Normalize(d);
        Debug.Log("d normalize =" + d);
		
        // dot product
        Vector3 x = new Vector3(1.0f, 3.0f, -4.0f);
        Vector3 y = new Vector3(2.0f, -5.0f, 8.0f);
        Debug.Log("x=" + x);
        Debug.Log("y=" + y);
        float i = Vector3.Dot(x, y);
        Debug.Log("x dot y =" + i);

        // cross product
        Vector3	j = Vector3.Cross(x, y);
        Debug.Log("x x y =" + j);
	    // angle between 2 vectors
	    Vector3 k = ComputeNormal(new Vector3(1.0f, 0.0f, 0.0f), new Vector3(1.0f, 1.0f, 0.0f), new Vector3(1.0f, 2.0f, 3.0f));
	    Debug.Log("Point (1, 0, 0) (1, 1, 0) (1, 2, 3) normal = " + k);
        // theta
        float angle = Mathf.Rad2Deg * Theta(new Vector3(1, 0, 0), new Vector3(0, 1, 0));
	    Debug.Log("angle between (1,0,0) and (0,1,0) = " + angle);
        // theta
        angle = Mathf.Rad2Deg * Theta(x, y);
	    Debug.Log("angle between (1,2,3) and (2,-5,8) = " + angle);
    }

    // Start is called before the first frame update
    void Start()
    {
        
    }

    // Update is called once per frame
    void Update()
    {

    }
}
